import re

from .. import utils

WORD_BLACKLIST = []
MAIN_PAGE = "Hauptseite"
HEADINGS = ['Überprüft']


def remove_i_tags(tree):
    for i in tree.xpath('//i'):
        i.getparent().remove(i)


def extract_quotes(tree, max_quotes):
    remove_i_tags(tree)

    q_lst = utils.extract_quotes_li(tree, max_quotes, HEADINGS, WORD_BLACKLIST)
    return [utils.remove_credit(q) for q in q_lst]


def qotd(html_tree):
    tree = html_tree.get_element_by_id('mf-ZitatdW')
    raw_text = tree.xpath('div')[1].text_content().strip()
    raw_text = re.sub(r'\(.*?\)', '', raw_text)

    raw_quote = []
    for part in raw_text.split('\n'):
        if part:
            raw_quote.append(part)
        if len(raw_quote) == 2:
            break

    return raw_quote[0], raw_quote[1]



import lxml.html

from . import utils
from . import langs
from .constants import DEFAULT_LANG


def quote_of_the_day(lang=DEFAULT_LANG):
    if lang not in langs.SUPPORTED_LANGUAGES:
        raise utils.UnsupportedLanguageException(
            'Unsupported language: ' + lang)

    main_page = langs.main_page_lang(lang)

    data = utils.json_from_url(utils.MAINPAGE_URL.format(lang=lang), main_page)
    html_tree = lxml.html.fromstring(data['parse']['text']['*'])
    return langs.qotd_lang(lang, html_tree)



import os
import glob
import importlib

from .. import utils

modules = glob.glob(os.path.dirname(__file__) + '/*.py')
modules = [m for m in modules if not m.endswith('__init__.py')]
modules = [os.path.basename(f)[:-3] for f in modules]

lang_dict = {}

for m in modules:
    module = importlib.import_module('.' + m, 'wikiquote.langs')
    lang_dict[m] = module


SUPPORTED_LANGUAGES = modules


def check_lang(lang):
    if lang not in SUPPORTED_LANGUAGES:
        raise utils.UnsupportedLanguageException(
            'Unsupported language: ' + lang)


def extract_quotes_lang(lang, html_tree, max_quotes):
    check_lang(lang)
    return lang_dict[lang].extract_quotes(html_tree, max_quotes)


def qotd_lang(lang, html_tree):
    check_lang(lang)
    return lang_dict[lang].qotd(html_tree)


def main_page_lang(lang):
    check_lang(lang)
    return lang_dict[lang].MAIN_PAGE



import lxml.etree

from .. import utils

MAIN_PAGE = "Portada"
WORD_BLACKLIST = ['Fuente:', 'Traducción:', 'Nota:']
HEADINGS = ['enlaces externos', 'referencias']


def extract_quotes(tree, max_quotes):
    return utils.extract_quotes_li(tree, max_quotes, HEADINGS, WORD_BLACKLIST)


def qotd(html_tree):
    tree = html_tree.get_element_by_id('mf-FDD')

    quote_container = tree.xpath('div/table/tbody/tr')
    raw_quote = quote_container[0].text_content().split('~')
    quote = raw_quote[0].strip()

    raw_author = quote_container[1].xpath('td/div/a')[0].text_content()
    author = raw_author.strip()

    return quote, author



import urllib.request
import urllib.parse
import json
import lxml
import re

from .constants import MIN_QUOTE_LEN, MIN_QUOTE_WORDS


class NoSuchPageException(Exception):
    pass


class DisambiguationPageException(Exception):
    pass


class UnsupportedLanguageException(Exception):
    pass


W_URL = 'http://{lang}.wikiquote.org/w/api.php'
SRCH_URL = W_URL + '?format=json&action=query&list=search&continue=&srsearch='
RANDOM_URL = W_URL + \
    '?format=json&action=query&list=random&rnnamespace=0&rnlimit={limit}'
PAGE_URL = W_URL + '?format=json&action=parse&prop=text|categories&' \
    'disableeditsection&page='
MAINPAGE_URL = W_URL + '?format=json&action=parse&prop=text&page='


def json_from_url(url, params=None):
    if params:
        url += urllib.parse.quote(params)
    res = urllib.request.urlopen(url)
    body = res.read().decode('utf-8')
    return json.loads(body)


def clean_txt(txt):
    # Remove unwanted characters
    txt = re.sub(r'«|»|"|“|”', '', txt)

    # Remove leading and trailing newlines/quotes
    return txt.strip()


def remove_credit(quote):
    if quote.endswith(('–', '-')):
        quote = quote[:-1].rstrip()
    return quote


def is_quote(txt, word_blacklist):
    txt_split = txt.split()
    invalid_conditions = [
        txt and txt[0].isalpha() and txt[0].islower(),
        len(txt) < MIN_QUOTE_LEN,
        len(txt_split) < MIN_QUOTE_WORDS,
        any(True for word in txt_split if word in word_blacklist),
        txt.endswith(('(', ':', ']')),
        txt.startswith(('(',))
    ]

    # Returns False if any invalid conditions are True, otherwise returns True.
    return not any(invalid_conditions)


def is_quote_node(node):
    # Discard nodes with the <small> tag
    if node.find('small') is not None:
        return False

    # Discard nodes that are just a link
    # (using xpath so lxml will show text nodes)
    # The link may be inside <i> or <b> tags, so keep peeling layers
    suspect_node = node
    while True:
        node_children = suspect_node.xpath('child::node()')
        if len(node_children) != 1:
            break

        suspect_node = node_children[0]
        if not isinstance(suspect_node, lxml.etree._Element):
            break

        if suspect_node.tag == 'a':
            return False

    return True


def extract_quotes_li(tree, max_quotes, headings, word_blacklist):
    # Check for quotes inside list items and description lists
    # This function works well for EN, DE and ES versions of Wikiquote articles
    quotes_list = []

    # Remove table of contents
    toc_list = tree.xpath('//div[@id="toc"]')
    for toc in toc_list:
        toc.getparent().remove(toc)

    # Scan for list items and description list tags
    # Also grab headlines to skip some sections.
    node_list = tree.xpath('//div/ul/li|//div/dl|//h2|//h3')

    # Skip all quotes above the first heading
    skip_to_next_heading = True

    for node in node_list:
        if node.tag not in ['h2', 'h3'] and skip_to_next_heading:
            continue

        if node.tag in ['h2', 'h3']:
            skip_to_next_heading = False
            heading_text = node.text_content().lower()

            # Commence skipping
            for unwanted_heading in headings:
                if heading_text.startswith(unwanted_heading.lower()):
                    skip_to_next_heading = True

            continue

        potential_quote = None

        if node.tag == 'dl':
            # <dl>'s are assumed to be multi-line dialogue
            dds = node.xpath('dd')

            if not all(is_quote_node(dd) for dd in dds):
                continue

            full_dialogue = '\n'.join(
                dd.text_content().strip()
                for dd in dds)

            potential_quote = clean_txt(full_dialogue)
        else:
            # Handle <li>'s
            uls = node.xpath('ul')
            for ul in uls:
                ul.getparent().remove(ul)

            if not is_quote_node(node):
                continue

            txt = ' '.join(node.text_content().split())
            potential_quote = clean_txt(txt)

        if potential_quote and is_quote(potential_quote, word_blacklist):
            quotes_list.append(potential_quote)
            if max_quotes == len(quotes_list):
                break

    return quotes_list



import lxml.etree

from .. import utils

MAIN_PAGE = "Pagina_principale"
WORD_BLACKLIST = []
HEADINGS = [
    'Bibliografia',
    'Opere',
    'Altri progetti',
    'Note',
    'Voci correlate'
]


def extract_quotes(tree, max_quotes):
    return utils.extract_quotes_li(tree, max_quotes, HEADINGS, WORD_BLACKLIST)


def qotd(html_tree):
    tree = html_tree.get_element_by_id('mf-Qotd')

    quote_container = tree.xpath('div')[0].text_content().split('„')

    quote = quote_container[0].lstrip('“').strip()
    author = quote_container[1].strip()

    return quote, author



from .quotes import quotes, random_titles, search
from .qotd import quote_of_the_day
from . import langs


def supported_languages():
    languages = langs.SUPPORTED_LANGUAGES[:]
    languages.sort()
    return languages



import lxml.etree

from .. import utils

WORD_BLACKLIST = ['quoted', 'Variant:', 'Retrieved', 'Notes:', 'article:']
MAIN_PAGE = "Main Page"
HEADINGS = ['cast', 'see also', 'external links', 'about']


def extract_quotes(tree, max_quotes):
    q_lst = utils.extract_quotes_li(tree, max_quotes, HEADINGS, WORD_BLACKLIST)
    return [utils.remove_credit(q) for q in q_lst]


def qotd(html_tree):
    tree = html_tree.get_element_by_id('mf-qotd')

    selector = 'div/div/table/tbody/tr'
    raw_quote = tree.xpath(selector)[0].text_content().split('~')
    quote = raw_quote[0].strip()
    author = raw_quote[1].strip()
    return quote, author



import lxml.html

from . import utils
from . import langs
from .constants import DEFAULT_LANG, DEFAULT_MAX_QUOTES


def _is_disambiguation(categories):
    # Checks to see if at least one category includes 'Disambiguation_pages'
    return not categories or any([
        category['*'] == 'Disambiguation_pages' for category in categories
    ])


def search(s, lang=DEFAULT_LANG):
    if lang not in langs.SUPPORTED_LANGUAGES:
        raise utils.UnsupportedLanguageException(
            'Unsupported language: ' + lang)

    if not s:
        return []

    local_srch_url = utils.SRCH_URL.format(lang=lang)
    data = utils.json_from_url(local_srch_url, s)
    results = [entry['title'] for entry in data['query']['search']]
    return results


def random_titles(lang=DEFAULT_LANG, max_titles=DEFAULT_MAX_QUOTES):
    if lang not in langs.SUPPORTED_LANGUAGES:
        raise utils.UnsupportedLanguageException(
            'Unsupported language: ' + lang)

    local_random_url = utils.RANDOM_URL.format(lang=lang, limit=max_titles)
    data = utils.json_from_url(local_random_url)
    results = [entry['title'] for entry in data['query']['random']]
    return results


def quotes(page_title, max_quotes=DEFAULT_MAX_QUOTES, lang=DEFAULT_LANG):
    if lang not in langs.SUPPORTED_LANGUAGES:
        raise utils.UnsupportedLanguageException(
            'Unsupported language: ' + lang)

    local_page_url = utils.PAGE_URL.format(lang=lang)
    data = utils.json_from_url(local_page_url, page_title)
    if 'error' in data:
        raise utils.NoSuchPageException(
            'No pages matched the title: ' + page_title)

    if _is_disambiguation(data['parse']['categories']):
        raise utils.DisambiguationPageException(
            'Title returned a disambiguation page.')

    html_content = data['parse']['text']['*']
    html_tree = lxml.html.fromstring(html_content)
    return langs.extract_quotes_lang(lang, html_tree, max_quotes)



from distutils.core import setup

with open('requirements.txt') as f:
    requires = f.read().splitlines()

setup(
    name='wikiquote',
    packages=['wikiquote', 'wikiquote.langs'],
    version='0.1.8',
    description='Retrieve quotes from any Wikiquote article.',
    author='Federico Tedin',
    author_email='federicotedin@gmail.com',
    install_requires=requires,
    url='https://github.com/federicotdn/python-wikiquotes',
    download_url='https://github.com/federicotdn/python-wikiquotes/archive/0.1.8.tar.gz',
    keywords=['quotes', 'wikiquote', 'python', 'api', 'qotd', 'quote', 'day'],
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: MIT License',
        'Topic :: Utilities'
    ]
)



from .. import utils

MAIN_PAGE = "Wikiquote:Accueil"


def extract_quotes(tree, max_quotes):
    # French wiki uses a "citation" HTML class
    nodes = tree.xpath('//div[@class="citation"]')
    quotes = [utils.clean_txt(node.text_content()) for node in nodes]
    return quotes[:max_quotes]


def qotd(html_tree):
    # French QOTD not available
    # TODO: Re-enable when QOTD appears again on welcome page
    # Note added on 2018/09/02
    raise utils.UnsupportedLanguageException(
        'Quote of the day not available for lang="fr".')
